/*****************************************************************************
 * misc-utils                                                                *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/crypt/blowfish.h                                         *
 * Created:     2001-05-21 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2001-2009 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by  the Free Software Foundation.                                         *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#ifndef BLOWFISH_H
#define BLOWFISH_H 1

#include "block64.h"


#define BFISH_ROUNDS 16

/* number of elements in the P array */
#define BFISH_SIZE_P (BFISH_ROUNDS + 2)

/* maximum key size in bytes */
#define BFISH_SIZE_KEY 56


/*!***************************************************************************
 * @short The blowfish context
 *****************************************************************************/
typedef struct {
	unsigned long P[BFISH_SIZE_P];
	unsigned long S[4][256];
} bfish_ctx_t;


/*****************************************************************************
 * @short Read a blowfish key from a byte buffer.
 * @param ctx The blowfish context
 * @param buf The byte buffer
 * @param n   The number of bytes in the buffer
 *
 * The maximum key length for blowfish is 56 bytes. If n > 56 then only
 * the first 56 bytes in the buffer are used and the remaining ignored.
 *****************************************************************************/
void bfish_set_key (bfish_ctx_t *ctx, const void *buf, unsigned n);

/*****************************************************************************
 * @short Invert an expanded blowfish key.
 * @param ctx The blowfish context
 *
 * After inverting the key, the context can be used for decryption.
 *****************************************************************************/
void bfish_invert_key (bfish_ctx_t *ctx);

/*****************************************************************************
 * @short Encrypt a block
 * @param ctx The blowfish context
 * @param blk The blocks to be encrypted
 * @param n   The number of blocks at blk
 *****************************************************************************/
void bfish_crypt (bfish_ctx_t *ctx, block64_t *blk, unsigned n);


#endif
