/*****************************************************************************
 * misc-utils                                                                *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/lib/adler32.c                                            *
 * Created:     2006-06-29 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2006-2009 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#include <lib/adler32.h>


void adler8_init (adler8_t *ad)
{
	ad->s1 = 1;
	ad->s2 = 0;
}

void adler8_copy (adler8_t *dst, const adler8_t *src)
{
	*dst = *src;
}

void adler8_calc (adler8_t *ad, const void *buf, unsigned long cnt)
{
	unsigned            i, n;
	const unsigned char *src = buf;

	while (cnt > 0) {
		ad->s1 %= 13;
		ad->s2 %= 13;

		n = (cnt > 4096) ? 4096 : cnt;

		for (i = 0; i < n; i++) {
			ad->s1 += *(src++);
			ad->s2 += ad->s1;
		}

		cnt -= n;
	}
}

void adler8_done (adler8_t *ad)
{
	ad->s1 %= 13;
	ad->s2 %= 13;
}

unsigned adler8_get_sum (adler8_t *ad)
{
	ad->s1 %= 13;
	ad->s2 %= 13;

	return ((ad->s2 << 4) | (ad->s1));
}


void adler16_init (adler16_t *ad)
{
	ad->s1 = 1;
	ad->s2 = 0;
}

void adler16_copy (adler16_t *dst, const adler16_t *src)
{
	*dst = *src;
}

void adler16_calc (adler16_t *ad, const void *buf, unsigned long cnt)
{
	unsigned            i, n;
	const unsigned char *src = buf;

	while (cnt > 0) {
		ad->s1 %= 251;
		ad->s2 %= 251;

		n = (cnt > 4096) ? 4096 : cnt;

		for (i = 0; i < n; i++) {
			ad->s1 += *(src++);
			ad->s2 += ad->s1;
		}

		cnt -= n;
	}
}

void adler16_done (adler16_t *ad)
{
	ad->s1 %= 251;
	ad->s2 %= 251;
}

unsigned adler16_get_sum (adler16_t *ad)
{
	ad->s1 %= 251;
	ad->s2 %= 251;

	return ((ad->s2 << 8) | (ad->s1));
}


void adler32_init (adler32_t *ad)
{
	ad->s1 = 1;
	ad->s2 = 0;
}

void adler32_copy (adler32_t *dst, const adler32_t *src)
{
	*dst = *src;
}

void adler32_calc (adler32_t *ad, const void *buf, unsigned long cnt)
{
	unsigned            i, n;
	const unsigned char *src = buf;

	while (cnt > 0) {
		ad->s1 %= 65521;
		ad->s2 %= 65521;

		n = (cnt > 4096) ? 4096 : cnt;

		for (i = 0; i < n; i++) {
			ad->s1 += *(src++);
			ad->s2 += ad->s1;
		}

		cnt -= n;
	}
}

void adler32_done (adler32_t *ad)
{
	ad->s1 %= 65521;
	ad->s2 %= 65521;
}

unsigned long adler32_get_sum (adler32_t *ad)
{
	ad->s1 %= 65521;
	ad->s2 %= 65521;

	return ((ad->s2 << 16) | (ad->s1));
}
