/*****************************************************************************
 * misc-utils                                                                *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/lib/mt19937.h                                            *
 * Created:     2006-08-16 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2006-2009 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#ifndef MISC_MT19937_H
#define MISC_MT19937_H 1


#define MT19937_N 624


typedef struct {
	unsigned      index;
	unsigned long state[MT19937_N];
} mt19937_t;


/*****************************************************************************
 * @short Initialize a mersenne twister
 * @param mt  The mersenne twister
 * @param val The seed value
 *****************************************************************************/
void mt_init_uint32 (mt19937_t *mt, unsigned long val);

/*****************************************************************************
 * @short Initialize a mersenne twister
 * @param mt   The mersenne twister
 * @param seed The seed array
 * @param cnt  The size of the seed array in bytes. 0 <= cnt <= 2496.
 *****************************************************************************/
void mt_init (mt19937_t *mt, const void *seed, unsigned cnt);

/*****************************************************************************
 * @short Copy a mersenne twister state
 * @param dst The destination
 * @param src The source
 *****************************************************************************/
void mt_copy (mt19937_t *dst, const mt19937_t *src);

/*****************************************************************************
 * @short Get random bits
 * @param  mt The mersenne twister
 * @return A random 32 bit value
 *****************************************************************************/
unsigned long mt_get_uint32 (mt19937_t *mt);

/*****************************************************************************
 * @short  Get a random number in the range [0.0, 1.0[
 * @param  mt The mersenne twister
 * @return A random number in the range [0.0, 1.0[
 *****************************************************************************/
double mt_get_double (mt19937_t *mt);


#endif
