/*****************************************************************************
 * misc-utils                                                                *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/lib/rmd160.h                                             *
 * Created:     2005-02-19 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2005-2009 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#ifndef MISC_RMD160_H
#define MISC_RMD160_H 1


/*!***************************************************************************
 * @short The RIPEMD-160 context
 *****************************************************************************/
typedef struct {
	unsigned long h[5];

	unsigned long cnt[2];

	unsigned char buf[64];
	unsigned      buf_i;
} rmd160_t;



/*!***************************************************************************
 * @short Initialize a RIPEMD-160 context
 *****************************************************************************/
void rmd160_init (rmd160_t *rmd);

/*!***************************************************************************
 * @short Copy a RIPEMD-160 context
 *****************************************************************************/
void rmd160_copy (rmd160_t *dst, const rmd160_t *src);

/*!***************************************************************************
 * @short Calculate a RIPEMD-160 message digest
 * @param rmd The RIPEMD-160 context
 * @param buf The data
 * @param cnt The number of bytes at buf
 *
 * This function can be called multiple times to add more bytes.
 *****************************************************************************/
void rmd160_calc (rmd160_t *rmd, const void *buf, unsigned long cnt);

/*!***************************************************************************
 * @short End calculating a RIPEMD-160 message digest
 *
 * This function must be called after the last call to rmd160_calc() and
 * before getting the message digest.
 *****************************************************************************/
void rmd160_done (rmd160_t *rmd);

/*!***************************************************************************
 * @short Get a 32bit word of the RIPEMD-160 message digest
 * @param rmd The RIPEMD-160 context
 * @param i   The word index. 0 <= i <= 4.
 *****************************************************************************/
unsigned long rmd160_get_sum_i (rmd160_t *rmd, unsigned i);

/*!***************************************************************************
 * @short Copy the RIPEMD-160 message digest to a byte buffer
 * @param rmd The RIPEMD-160 context
 * @param buf The 20 byte RIPEMD-160 buffer
 *****************************************************************************/
void rmd160_get_sum (rmd160_t *rmd, void *buf);

/*!***************************************************************************
 * @short Calculate the RIPEMD-160 message digest of a buffer
 * @param dst The 20 byte RIPEMD-160 buffer
 * @param src The source buffer
 * @param n   The size in bytes of the source buffer
 *****************************************************************************/
void rmd160_calc1 (void *dst, const void *src, unsigned long n);


#endif
