/*****************************************************************************
 * misc-utils                                                                *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/lib/stresc.c                                             *
 * Created:     2004-03-07 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2004-2009 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#include <lib/stresc.h>


static
unsigned char esc_backslash (unsigned char c)
{
	switch (c) {
		case '\\':
			return ('\\');

		case 'a':
			return ('\a');

		case 'b':
			return ('\b');

		case '\r':
			return ('\r');

		case '\t':
			return ('\t');

		case '\n':
			return ('\n');
	}

	return (c);
}

unsigned buf_escape (unsigned char *d, const unsigned char *s, unsigned max)
{
	unsigned      cnt;
	unsigned char c;

	cnt = 0;

	while ((cnt < max) && (*s != 0)) {
		c = 0;

		if (*s == '\\') {
			s += 1;

			if (*s == '0') {
				s += 1;

				while ((*s >= '0') && (*s <= '7')) {
					c = (8 * c) + (*s - '0');
					s += 1;
				}
			}
			else if ((*s > '0') && (*s <= '9')) {
				c = *(s++) - '0';

				while ((*s >= '0') && (*s <= '9')) {
					c = 10 * c + (*s - '0');
					s += 1;
				}
			}
			else if ((*s == 'x') || (*s == 'X')) {
				while (1) {
					s += 1;

					if ((*s >= '0') && (*s <= '9')) {
						c = (16 * c) + (*s - '0');
					}
					else if ((*s >= 'a') && (*s <= 'f')) {
						c = (16 * c) + (*s - 'a' + 10);
					}
					else if ((*s >= 'A') && (*s <= 'F')) {
						c = (16 * c) + (*s - 'A' + 10);
					}
					else {
						break;
					}
				}
			}
			else {
				c = esc_backslash (*(s++));
			}
		}
		else {
			c = *s;
			s += 1;
		}

		*(d++) = c;

		cnt += 1;
	}

	return (cnt);
}

int str_escape (char *dst, const char *src, unsigned max)
{
	unsigned n;

	n = 0;
	while (src[n] != 0) {
		n += 1;
	}

	n = buf_escape ((unsigned char *) dst, (const unsigned char *) src, max);

	if (n >= max) {
		return (1);
	}

	dst[n] = 0;

	return (0);
}
