#!/bin/bash

case "$1" in
"-?" | "--help")
	echo "usage: lndir [options] src"
	echo "  --help          Print usage information"
	echo "  --version       Print version information"
	echo "  -v, --verbose   Verbose operation"
	echo "  -h, --hardlink  Use hard links [no]"
	echo "  -u, --unlink    Unlink existing files [no]"
	exit 0
	;;
"--version")
	echo "lndir (misc-utils) version %MISC_VERSION_STR"
	echo ""
	echo "Copyright (C) 2000-2009 by Hampa Hug <hampa@hampa.ch>"
	exit 0
	;;
esac


# prt_string level text
prt_string ()
{
	if test $par_verbose -ge $1 ; then
		echo "$2"
	fi
}

# lnk_link src dst
lnk_link ()
{
	local src=$1
	local dst=$2

	if test "x$par_hard" = "x1" ; then
		ln "$src" "$dst"
	else
		ln -s "$src" "$dst"
	fi
}

# lnk_relink dst
lnk_relink ()
{
	local src
	local dst="$1"
	local f

	src=$(readlink "$dst")

	rm -f "$dst"
	mkdir -p "$dst"

	if test ! -e "$src" ; then
		return
	fi

	case "$src" in
	/*)
		;;
	*)
		src="../$src"
		;;
	esac

	(
		cd "$dst"
		for f in "$src"/* ; do
			if test -e "$f" ; then
				lnk_link "$f" .
			fi
		done
	)
}

# lnk_dir src
# link directory <src> into current directory
lnk_dir ()
{
	local src="$1"
	local dst
	local bak

	dst=$(basename "$src")

	if test -e "$dst" ; then
		if test ! -d "$dst" ; then
			if test "x$par_unlink" = "x1" ; then
				prt_string 1 "unlink  ${par_rel}$dst"
				rm -f "$dst"
			else
				prt_string 2 "skip    ${par_rel}$dst <- $src"
				return 0
			fi
		fi
	else
		if test -L "$dst" ; then
			prt_string 1 "replace ${par_rel}$dst <- $src"
			rm -f "$dst"
		fi
	fi

	if test "x$par_hard" = "x1" ; then
		if test ! -e "$dst" ; then
			mkdir -p "$dst"
		fi
	fi

	if test -d "$dst" ; then
		if test -L "$dst" ; then
			if test "x$par_hard" = "x1" ; then
				prt_string 1 "relink  ${par_rel}$dst"
				lnk_relink "$dst"
			else
				lnk=$(readlink "$dst")
				if test "$lnk" != "$src" ; then
					prt_string 1 "relink  ${par_rel}$dst"
					lnk_relink "$dst"
				else
					return 0
				fi
			fi
		fi

		cd "$dst"
		bak=$par_rel
		par_rel="${par_rel}$dst/"
		case "$src" in
		/*)
			lndir "$src"
			;;
		*)
			lndir "../$src"
			;;
		esac
		par_rel=$bak
		cd ..
	else
		prt_string 1 "link    ${par_rel}$dst <- $src"
		ln -s "$src" "$dst"
	fi
}

# lnk_file src
# link file <src> into current directory
lnk_file ()
{
	local src="$1"
	local dst

	dst=$(basename "$src")

	if test -d "$dst" ; then
		echo "$0: $dst is a directory"
	elif test -L "$dst" -a ! -e "$dst" ; then
		prt_string 1 "replace $dst <- $src"
		rm -f "$dst"
		lnk_link "$src" "$dst"
	elif test -e "$dst" ; then
		if test "x$par_unlink" = "x1" ; then
			prt_string 1 "unlink  $dst"
			rm -f "$dst"

			prt_string 1 "link    ${par_rel}$dst <- $src"
			lnk_link "$src" "$dst"
		else
			prt_string 2 "skip    ${par_rel}$dst <- $src"
		fi
	else
		prt_string 1 "link    ${par_rel}$dst <- $src"
		lnk_link "$src" "$dst"
	fi
}

# lndir src
lndir () {
	local src=$1
	local s

	prt_string 3 "enter   $(pwd)"

	case "$src" in
	"/")
		;;
	*)
		src=${src%/}
		;;
	esac

	for s in "$src"/* ; do
		if test ! -e "$s" ; then
			continue
		fi

		if test -d "$s" ; then
			lnk_dir "$s"
		else
			lnk_file "$s"
		fi
	done

	prt_string 3 "leave   $(pwd)"
}

par_verbose=0
par_unlink=0
par_hard=0
par_rel=""

while test $# -gt 0 ; do
	case "$1" in
	"-v" | "--verbose")
		par_verbose=$(($par_verbose+1))
		;;
	"-vv")
		par_verbose=$((par_verbose+2))
		;;
	"-u" | "--unlink")
		par_unlink=1
		;;
	"-h" | "--hardlink")
		par_hard=1
		;;
	-*)
		echo "$0: unknown option ($1)" >&2
		exit 1
		;;
	*)
		if test -d "$1" ; then
			lndir "$1"
		else
			lnk_file "$1"
		fi
		;;
	esac
	shift 1
done
