/*****************************************************************************
 * misc-utils                                                                *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/readlink/readlink.c                                      *
 * Created:     2003-04-07 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2002-2009 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#include <lib/common.h>
#include <lib/getopt.h>

#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include "readlink.h"

#ifndef PATH_MAX
#define PATH_MAX 256
#endif


static const char *argv0 = NULL;

static int par_nl = 1;
static int par_realpath = 0;


static misc_option_t opts[] = {
	{ '?', 0, "help", NULL, "Print usage information" },
	{ 'f', 0, "canonicalize", NULL, "Follow every element of a path" },
	{ 'n', 0, "no-newline", NULL, "Don't print a newline after the link" },
	{ 'V', 0, "version", NULL, "Print version information" },
	{  -1, 0, NULL, NULL, NULL }
};


static
void print_help (void)
{
	misc_getopt_help (
		"readlink: show the contents of symlinks",
		"usage: readlink [options] [link...]",
		opts
	);

	fflush (stdout);
}

static
int do_readlink (const char *name)
{
	int  n;
	char buf[PATH_MAX];

	n = readlink (name, buf, PATH_MAX);
	if (n < 0) {
		return (1);
	}

	if (fwrite (buf, 1, n, stdout) != n) {
		return (1);
	}

	if (par_nl) {
		fputs ("\n", stdout);
	}

	fflush (stdout);

	return (0);
}

static
int do_realpath (const char *name)
{
	char *ret;
	char buf[PATH_MAX];

	ret = realpath (name, buf);

	if (ret == NULL) {
		return (1);
	}

	fputs (ret, stdout);

	if (par_nl) {
		fputs ("\n", stdout);
	}

	fflush (stdout);

	return (0);
}

int main (int argc, char *argv[])
{
	int  r;
	char **optarg;

	argv0 = argv[0];

	while (1) {
		r = misc_getopt (argc, argv, &optarg, opts);

		if (r == GETOPT_DONE) {
			break;
		}

		if (r < 0) {
			return (EXIT_FAILURE);
		}

		switch (r) {
		case '?':
			print_help();
			return (EXIT_SUCCESS);

		case 'V':
			prt_version ("readlink");
			return (EXIT_SUCCESS);

		case 'f':
			par_realpath = 1;
			break;

		case 'n':
			par_nl = 0;
			break;

		case 0:
			if (par_realpath) {
				if (do_realpath (optarg[0])) {
					return (EXIT_FAILURE);
				}
			}
			else {
				if (do_readlink (optarg[0])) {
					return (EXIT_FAILURE);
				}
			}
			break;

		default:
			return (EXIT_FAILURE);
		}
	}

	return (EXIT_SUCCESS);
}
